@@ -42,7 +42,7 @@ static PyTypeObject Bitarraytype;
 
 #define BYTES(bits)  (((bits) == 0) ? 0 : (((bits) - 1) / 8 + 1))
 
-#define BITMASK(endian, i)  (((char) 1) << ((endian) ? 7 - (i)%8 : (i)%8))
+#define BITMASK(endian, i)  (((char) 1) << ((endian) ? (7 - (i)%8) : (i)%8))
 
 /* ------------ Low level access to bits in bitarrayobject ------------- */
 
@@ -68,17 +68,15 @@ setbit(bitarrayobject *self, idx_t i, int bit)
 static int
 check_overflow(idx_t nbits)
 {
-    idx_t maxbit;
+    idx_t max_bits;
 
     assert (nbits >= 0);
-    maxbit = PY_SSIZE_T_MAX;
-
-    if (sizeof(void *) == 4) { /* 32bit machine */
-        maxbit *= 8;
-	if (nbits > maxbit) {
+    if (sizeof(void *) == 4) {  /* 32bit machine */
+        max_bits = ((idx_t) 1) << 34;  /* 2^34 = 16 Gbits*/
+	if (nbits > max_bits) {
 	    char buff[256];
 	    sprintf(buff, "cannot create bitarray of size %lld, "
-		          "max size is %lld", nbits, maxbit);
+		          "max size is %lld", nbits, max_bits);
 	    PyErr_SetString(PyExc_OverflowError, buff);
 	    return -1;
 	}
@@ -189,8 +187,8 @@ static void
 bitcopy(bitarrayobject *self, idx_t a,
         bitarrayobject *other, idx_t b, idx_t n)
 {
-    /* XXX: This function can be highly optimized using memcpy 
-       when self and other have same endianness, and other != self
+    /* XXX: This function could be highly optimized using memcpy for some
+       cases when self and other have same endianness, and other != self.
     */
     idx_t i;
     
@@ -226,7 +224,9 @@ delete(bitarrayobject *self, idx_t start, idx_t n)
     return 0;
 }
 
-/* at start, insert n (arbitrary) bits into self */
+/* at start, insert n bits into self.
+   The vaules of the inserted bits are unspecified.
+*/
 static int
 insert(bitarrayobject *self, idx_t start, idx_t n)
 {
@@ -327,15 +327,11 @@ bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
 static void
 setrange(bitarrayobject *self, idx_t start, idx_t stop, int val)
 {
-    /* XXX: This function can be highly optimized using memset */
     idx_t i;
-    
-    assert (start >= 0 && start <= self->nbits);
-    assert (stop  >= 0 && stop  <= self->nbits);
-    assert (start <= stop);
-    
+
+    assert (start <= self->nbits && stop <= self->nbits);
     for (i = start; i < stop; i++)
-        setbit(self, i, val);
+	setbit(self, i, val);
 }
 
 /* reverse the order of bits in each byte of the buffer */
@@ -345,6 +341,9 @@ bytereverse(bitarrayobject *self)
     static char trans[256];
     static int setup = 0;
     if (!setup) {
+	/* Setup a translation table, which maps each byte to it's
+	   reversed: trans = {0, 128, 64, 192, 32, 160, ..., 255}
+        */
         int j, k;
         for (k = 0; k < 256; k++) {
             trans[k] = '\0';
@@ -356,7 +355,7 @@ bytereverse(bitarrayobject *self)
     }
     Py_ssize_t i;
     int c;
-        
+
     setunused(self);
     for (i = 0; i < self->ob_size; i++) {
         c = self->ob_item[i];
@@ -379,14 +378,17 @@ invert(bitarrayobject *self)
 static idx_t
 count(bitarrayobject *self)
 {
-    static int bytecount[256];
+    static int bitcount[256];
     static int setup = 0;
     if (!setup) {
+        /* Setup a translation table, which maps each byte to it's
+	   bit count: trans = {0, 1, 1, 2, 1, 2, 2, 3, 1, ..., 8}
+        */
         int j, k;
         for (k = 0; k < 256; k++) {
-            bytecount[k] = 0;
+            bitcount[k] = 0;
             for(j = 0; j < 8; j++)
-                bytecount[k] += (k >> j) & 1;
+                bitcount[k] += (k >> j) & 1;
         }
         setup = 1;
     }
@@ -400,7 +402,7 @@ count(bitarrayobject *self)
         c = self->ob_item[i];
         /* in case conversion to int puts a sign in */
         if (c < 0) c += 256;
-        res += bytecount[c];
+        res += bitcount[c];
     }
     return res;
 }
@@ -442,7 +444,7 @@ set_item(bitarrayobject *self, idx_t i, PyObject *v)
     assert (i >= 0 && i < self->nbits);
     if ((vi = PyObject_IsTrue(v)) < 0)
 	return -1;
-    
+
     setbit(self, i, vi);
     return 0;
 }
@@ -764,12 +766,12 @@ Return a copy of the bitarray.");
 
 
 static PyObject *
-bitarray_count(bitarrayobject *self, PyObject *v)
+bitarray_count(bitarrayobject *self, PyObject *args)
 {
     idx_t n1;
-    long vi;
-    
-    if ((vi = PyObject_IsTrue(v)) < 0)
+    long vi = 1;
+
+    if (!PyArg_ParseTuple(args, "|i:count", &vi))
         return NULL;
 
     n1 = count(self);
@@ -777,9 +779,9 @@ bitarray_count(bitarrayobject *self, PyObject *v)
 }
 
 PyDoc_STRVAR(count_doc,
-"count(x)\n\
+"count([x])\n\
 \n\
-Return number of occurrences of x in the bitarray.");
+Return number of occurrences of x in the bitarray.  x defaults to True.");
 
 
 static PyObject *
@@ -962,19 +964,16 @@ static PyObject *
 bitarray_reduce(bitarrayobject *self)
 {
     PyObject *dict, *result;
-    int unusedbits;
-    
+
     dict = PyObject_GetAttrString((PyObject *) self, "__dict__");
     if (dict == NULL) {
         PyErr_Clear();
         dict = Py_None;
         Py_INCREF(dict);
     }
-    unusedbits = setunused(self);
-    result = Py_BuildValue("O(s#is)O",
+    result = Py_BuildValue("O(Os)O",
                            self->ob_type,
-                           self->ob_item, self->ob_size,
-			   unusedbits,
+                           unpack(self, '0', '1'),
 			   ENDIANSTR(self->endian),
                            dict);
     Py_DECREF(dict);
@@ -1123,11 +1122,10 @@ static PyObject *
 bitarray_fromfile(bitarrayobject *self, PyObject *args)
 {
     PyObject *f;
-    Py_ssize_t nbytes;
+    Py_ssize_t nbytes = -1;
     FILE *fp;
     long cur;
 
-    nbytes = -1;
     if (!PyArg_ParseTuple(args, "O|n:fromfile", &f, &nbytes))
         return NULL;
     
@@ -1429,7 +1427,7 @@ bitarray_remove(bitarrayobject *self, PyObject *v)
 {
     idx_t i;
     long vi;
-    
+
     vi = PyObject_IsTrue(v);
     if (vi < 0)
         return NULL;
@@ -1493,8 +1491,8 @@ bitarray_setitem(bitarrayobject *self, PyObject *args)
 {
     PyObject *a, *v;
     idx_t start, stop, step, slicelength, i, j;
-    
-    if (!PyArg_ParseTuple(args, "OO|__setitem__", &a, &v))
+
+    if (!PyArg_ParseTuple(args, "OO:__setitem__", &a, &v))
         return NULL;
     
     if (ISINDEX(a)) {
@@ -1884,9 +1882,10 @@ bitarray_decode(bitarrayobject *self, PyObject *codedict)
     while (PyDict_Next(codedict, &i, &symbol, &ba))
 	if (btree_insert(tree, symbol, ba) < 0)
 	    goto finish;
-    
-    /* XXX printf("tree = %s\n", PyString_AsString(PyObject_Repr(tree))); */
-    
+#if 0
+    /* debug binary tree */
+    printf("tree = %s\n", PyString_AsString(PyObject_Repr(tree)));
+#endif
     iter = PyObject_GetIter((PyObject *) self);
     if (iter == NULL)
 	goto finish;
@@ -1899,9 +1898,6 @@ bitarray_decode(bitarrayobject *self, PyObject *codedict)
 			    "prefix code does not match data in bitarray");
 	    Py_DECREF(res);
 	    res = NULL;
-	    /* XXX res sould contain symbols read so far, but then
-	       res is not NULL an no exception is raised.
-	    */
 	    goto finish;
 	}
 	if (PyList_Append(res, symbol) < 0)
@@ -1937,7 +1933,7 @@ bitarray_methods[] = {
      bytereverse_doc},
     {"copy",         (PyCFunction) bitarray_copy,        METH_NOARGS,
      copy_doc},
-    {"count",        (PyCFunction) bitarray_count,       METH_O,
+    {"count",        (PyCFunction) bitarray_count,       METH_VARARGS,
      count_doc},
     {"decode",       (PyCFunction) bitarray_decode,      METH_O,
      decode_doc},
@@ -2026,14 +2022,13 @@ static PyObject *
 bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
     PyObject *initial = NULL;
-    int unusedbits = -1;
     char *endianStr = "<NOT_PROVIDED>";
     int endian = DEFAULT_ENDIAN;
-    static char* kwlist[] = {"initial", "unusedbits", "endian", NULL};
+    static char* kwlist[] = {"initial", "endian", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds,
-                                     "|Ois:bitarray", kwlist,
-                                     &initial, &unusedbits, &endianStr)) {
+                                     "|Os:bitarray", kwlist,
+                                     &initial, &endianStr)) {
         return NULL;
     }
     if (strcmp(endianStr, "little") == 0) {
@@ -2081,36 +2076,14 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         return a;
     }
 
-    /* the following cases extend a */
+    /* leave remaining type dispatch to the extend method */
     a = newbitarrayobject(type, 0, endian);
     if (a == NULL)
 	return NULL;
 
-    /* raw string, for pickling and expert use only */
-    if (PyString_Check(initial) && unusedbits >= 0) {
-        if (unusedbits >= 8) {
-            PyErr_SetString(PyExc_ValueError, "unusedbits not 0..7");
-            return NULL;
-        }
-        if (extend_rawstring((bitarrayobject *) a, initial) < 0)
-            return NULL;
-        
-#define abits  (((bitarrayobject *) a)->nbits)
-        assert (abits % 8 == 0);
-        if (abits == 0 && unusedbits > 0) {
-            PyErr_SetString(PyExc_ValueError, 
-                            "unusedbits > 0 given but string is empty");
-            return NULL;
-        }
-        abits -= unusedbits;
-#undef abits
-        return a;
-    }
-
-    /* leave remaining type  dispatch to the extend method */
     if (bitarray_extend((bitarrayobject *) a, initial) == NULL)
         return NULL;
-    
+
     return a;
 }
 
