@@ -30,7 +30,7 @@ class Util(object):
             yield bitarray([randint(0, 1) for d in xrange(n)],
                            endian='big' if randint(0, 1) else 'little')
 
-    def randomlists(self, N=500):
+    def randomlists(self):
         for n in range(25) + [randint(1000, 2000)]:
             yield [bool(randint(0, 1)) for d in xrange(n)]
 
@@ -227,25 +227,6 @@ class CreateObjectTests(unittest.TestCase, Util):
             self.check_obj(a)
             
         self.assertRaises(ValueError, bitarray.__new__, bitarray, '01012100')
-        
-
-    def test_string(self):
-        a = bitarray('', 0)
-        self.assertEqual(a.tolist(), [])
-        self.check_obj(a)
-        
-        for i in xrange(8):
-            self.assertEqual(bitarray('\xff', i).tolist(), (8 - i)*[True])
-            self.assertEqual(bitarray('\x00', i).tolist(), (8 - i)*[False])
-
-        a = bitarray('\xff\x00', 0)
-        self.assertEqual(a.tolist(), 8*[True] + 8*[False])
-        
-        for n in xrange(50):
-            a = bitarray(n*'A', 0)
-            self.assertEqual(a.tostring(), n*'A')
-            self.assertEqual(len(a), 8*n)
-            self.check_obj(a)
 
 
     def test_bitarray(self):
@@ -268,29 +249,19 @@ class CreateObjectTests(unittest.TestCase, Util):
     def test_None(self):
         self.assertEQUAL(bitarray(), bitarray(0))
         self.assertEQUAL(bitarray(), bitarray(None))
-        
-        for i in xrange(-10, 20):
-            self.assertEQUAL(bitarray(None, i), bitarray())
 
 
     def test_WrongArgs(self):
         self.assertRaises(TypeError, bitarray.__new__, bitarray, 'A', 42, 69)
-        
+
         self.assertRaises(TypeError, bitarray.__new__, bitarray, Ellipsis)
         self.assertRaises(TypeError, bitarray.__new__, bitarray, slice(0))
-        
-        
+
         self.assertRaises(TypeError, bitarray.__new__, bitarray, 2.345)
         self.assertRaises(TypeError, bitarray.__new__, bitarray, 4+3j)
-            
-        for i in xrange(1, 20):
-            self.assertRaises(ValueError, bitarray.__new__, bitarray, '', i)
 
-        for i in xrange(8, 20):
-            self.assertRaises(ValueError, bitarray.__new__, bitarray, 'a', i)
-        
         self.assertRaises(TypeError, bitarray.__new__, bitarray, '', 0, 42)
-        self.assertRaises(ValueError, bitarray.__new__, bitarray, '', 0, 'foo')
+        self.assertRaises(ValueError, bitarray.__new__, bitarray, 0, 'foo')
 
         
 tests.append(CreateObjectTests)
@@ -555,15 +526,25 @@ class MiscTests(unittest.TestCase, Util):
                 self.assertEqual(a[i], b[i+1234])
 
     def test_endianness(self):
-        self.assertEqual(bitarray('\x01', 0, 'little').to01(), '10000000')
-        self.assertEqual(bitarray('\x80', 0, 'little').to01(), '00000001')
+        a = bitarray(endian='little')
+        a.fromstring('\x01')
+        self.assertEqual(a.to01(), '10000000')
         
-        self.assertEqual(bitarray('\x80', 0, 'big').to01(), '10000000')
-        self.assertEqual(bitarray('\x01', 0, 'big').to01(), '00000001')
-        
-        self.assertEqual(bitarray('\x01', 0, 'little'),
-                         bitarray('\x80', 0, 'big'))
+        b = bitarray(endian='little')
+        b.fromstring('\x80')
+        self.assertEqual(b.to01(), '00000001')
+
+        c = bitarray(endian='big')
+        c.fromstring('\x80')
+        self.assertEqual(c.to01(), '10000000')
         
+        d = bitarray(endian='big')
+        d.fromstring('\x01')
+        self.assertEqual(d.to01(), '00000001')
+
+        self.assertEqual(a, c)
+        self.assertEqual(b, d)
+
         a = bitarray(8, endian='little')
         a.setall(False)
         a[0] = True
@@ -611,6 +592,18 @@ class MiscTests(unittest.TestCase, Util):
             self.assert_(b is not a)
             self.assertEQUAL(a, b)
 
+    def test_overflow(self):
+        from platform import architecture
+
+        if architecture()[0] == '64bit':
+            return
+
+        self.assertRaises(OverflowError, bitarray.__new__,
+                          bitarray, 2**34 + 1)
+
+        a = bitarray(10**6)
+        self.assertRaises(OverflowError, a.__imul__, 17180)
+        
 
 tests.append(MiscTests)
 
@@ -1076,11 +1069,15 @@ class MethodTests(unittest.TestCase, Util):
 
     def test_count(self):
         a = bitarray('10011')
+        self.assertEqual(a.count(), 3)
         self.assertEqual(a.count(True), 3)
         self.assertEqual(a.count(False), 2)
-        self.assertEqual(a.count('foo'), 3)
-        self.assertEqual(a.count({}), 2)
+        self.assertEqual(a.count(1), 3)
+        self.assertEqual(a.count(0), 2)
+        self.assertRaises(TypeError, a.count, 'A')
+
         for a in self.randombitarrays():
+            self.assertEqual(a.count(), a.count(1))
             self.assertEqual(a.count(1), a.to01().count('1'))
             self.assertEqual(a.count(0), a.to01().count('0'))
 
@@ -1126,6 +1123,10 @@ class MethodTests(unittest.TestCase, Util):
         a.sort(reverse=True)
         self.assertEqual(a, bitarray('1110000'))
 
+        a = bitarray('1101000')
+        a.sort(True)
+        self.assertEqual(a, bitarray('1110000'))
+
         self.assertRaises(TypeError, a.sort, 'A')
         
         for a in self.randombitarrays():
@@ -1377,7 +1378,6 @@ class FileTests(unittest.TestCase, Util):
     def setUp(self):
         self.tmpdir = tempfile.mkdtemp()
         self.tmpfname = os.path.join(self.tmpdir, 'testfile')
-        
 
     def tearDown(self):
         shutil.rmtree(self.tmpdir)
@@ -1397,10 +1397,28 @@ class FileTests(unittest.TestCase, Util):
             self.assertEQUAL(a, b)
 
 
+    def test_shelve(self):
+        import shelve, md5
+
+        d = shelve.open(self.tmpfname)
+        stored = []
+        for a in self.randombitarrays():
+            key = md5.md5(repr(a) + a.endian()).hexdigest()
+            d[key] = a
+            stored.append((key, a))
+        d.close()
+        del d
+
+        d = shelve.open(self.tmpfname)
+        for k, v in stored:
+            self.assertEQUAL(d[k], v)
+        d.close()
+
+
     def test_fromfile_wrong_args(self):
         b = bitarray()
         self.assertRaises(TypeError, b.fromfile)
-        self.assertRaises(TypeError, b.fromfile, StringIO()) # not open
+        self.assertRaises(TypeError, b.fromfile, StringIO()) # file not open
         self.assertRaises(TypeError, b.fromfile, 42)
         self.assertRaises(TypeError, b.fromfile, 'bar')
 
